import unittest

import ghtalks

import os

class TestIndex(unittest.TestCase):


    expected_output="""
<body>\n<div class="page-content">\n<div class="wrap">\n<div class="home">\n<ul class=\'posts\'>\n<li>\n<span class="post-date">2015-07-27 [.link]</span>\n<a class="post-link" href="http://vknight.org/Talks/\n">links</a>\n\n</li>\n<li>\n<span class="post-date">2015-04-05 [.html]</span>\n<a class="post-link" href="./examples/2015-04-05-Organise-and-share-talks/index.html">Organise and share talks</a>\n<ul>\n<li><a href=http://lab.hakim.se/reveal-js/#/>(slides written in reveal.js)</a></li>\n\n<li><a href=https://github.com/drvinceknight/ghtalks>github repository</a></li>\n</ul>\n</li>\n<li>\n<span class="post-date">2014-12-25 [.pdf]</span>\n<a class="post-link" href="./examples/archive/2014-12-25-Auraya/index.pdf">Auraya</a>\n\n</li>\n</ul>\n</div>\n</div>\n</div>\n\n</body>\n"""

    def test_init(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        crwl.crawl_directories(os.path.join(root, 'examples'))
        index = ghtalks.Index(crwl.generate_talks())
        talk_titles = sorted([t.title for t in index.talks])
        talk_dates = sorted([t.date for t in index.talks])
        talk_paths = sorted([t.path for t in index.talks])
        self.assertEqual(talk_titles, ['Auraya', 'Organise and share talks',
            'links'])
        self.assertEqual(talk_dates, ['2014-12-25', '2015-04-05', '2015-07-27'])
        self.assertEqual(talk_paths,
                ['./examples/2015-04-05-Organise-and-share-talks/index.html',
                    './examples/archive/2014-12-25-Auraya/index.pdf',
                    'http://vknight.org/Talks/\n'])

    def test_generate_output(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        crwl.crawl_directories(os.path.join(root, 'examples'))
        index = ghtalks.Index(crwl.generate_talks())
        index.generate_output()

        self.assertEqual(index.out, self.expected_output)

    def test_write(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        crwl.crawl_directories(os.path.join(root, 'examples'))
        index = ghtalks.Index(crwl.generate_talks())
        index.generate_output()
        index.write()
        self.assertEqual(open('index.html', 'r').read(), self.expected_output)
        os.remove('index.html')



"""
Data holder for a talk
"""
import yaml


class Talk:
    def __init__(self, title, date, path, file_type, urls=None):
        self.title = title
        self.date = date
        self.path = path
        self.file_type = file_type
        self.urls = urls

    def html(self):
        out = """
<span class="post-date">%s [%s]</span>
<a class="post-link" href="%s">%s</a>
""" % (self.date, self.file_type, self.path, self.title)
        if self.urls:
            out += "<ul>"
            urls = yaml.load(open(self.urls, 'r'))
            for key in sorted(urls.keys()):
                out += """
<li><a href=%s>%s</a></li>
""" % (urls[key], key)
            out += "</ul>"
        return out



import unittest

import ghtalks

import os


class TestCrawl(unittest.TestCase):

    def test_init(self):
        root = os.getcwd()
        directory = os.path.join(root, '.')
        crwl = ghtalks.Crawl()
        self.assertEquals(crwl.directory, directory)

        crwl = ghtalks.Crawl('a_directory')
        directory = os.path.join(root, 'a_directory')
        self.assertEquals(crwl.directory, directory)

    def test_crawl_directory(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        crwl.crawl_directories(os.path.join(root, 'examples'))
        self.assertEquals(sorted(crwl.directories), [os.path.join(root, d)
                          for d in
                          ['examples/2015-04-05-Organise-and-share-talks',
                                    'examples/archive/2014-12-25-Auraya',
                                    'examples/archive/2015-07-27-links']])

    def test_has_date(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        self.assertTrue(crwl.has_date('2015-03-04'))
        self.assertTrue(
            crwl.has_date(os.path.join(root, '2015-12-04-directory')))
        self.assertFalse(
            crwl.has_date(os.path.join(root, '2015-12-32-directory')))
        self.assertFalse(
            crwl.has_date(os.path.join(root, '15-12-04-directory')))
        self.assertFalse(
            crwl.has_date(os.path.join(root, '1983-13-04-directory')))
        self.assertFalse(crwl.has_date(os.path.join(root, 'directory')))
        self.assertFalse(
            crwl.has_date(os.path.join(root, 'directory_2015-12-04')))

    def test_generate_talks(self):
        root = os.getcwd()
        crwl = ghtalks.Crawl()
        crwl.crawl_directories(os.path.join(root, 'examples'))
        talks = crwl.generate_talks()
        talk_titles = sorted([t.title for t in talks])
        talk_dates = sorted([t.date for t in talks])
        talk_paths = sorted([t.path for t in talks])
        self.assertEqual(talk_titles,
                         ['Auraya', 'Organise and share talks', 'links'])
        self.assertEqual(talk_dates, ['2014-12-25', '2015-04-05', '2015-07-27'])
        self.assertEqual(talk_paths,
                         ['./examples/2015-04-05-Organise-and-share-talks/index.html',
                          './examples/archive/2014-12-25-Auraya/index.pdf',
                          'http://vknight.org/Talks/\n'])



"""
Crawl a given directory to find files of required type
"""
import os
import glob

import re
import talk


class Crawl:

    """
    A class for the complete crawl of a directory
    """

    def __init__(self, directory='.'):
        self.root = os.getcwd()
        self.directory = os.path.join(self.root, directory)
        # Below is a regex for a date in ISO global date format
        self.date_regex = '(19|20)\d\d[- ./](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])'

    def crawl_directories(self, directory):
        self.directories = [f[0] for f in os.walk(directory) if
                            (os.path.isdir(f[0]) and self.has_date(f[0]))]

    def has_date(self, directory):
        return bool(re.search(self.date_regex,
            os.path.basename(os.path.normpath(directory))[:len('yyyy-mm-dd')]))

    def generate_talks(self, filetypes=['.pdf', '.html', '.link'],
                       talk_filename='index'):
        self.talks = []
        for d in self.directories:
            d = d[len(self.root):]
            try:
                tlk = [f for f in glob.glob('.' + d + '/' + talk_filename +
                    '.*') if any(ext in f for ext in filetypes)][0]
                raw_title, extension = os.path.splitext(tlk)
                date = re.search(self.date_regex, d).group()
                title = os.path.basename(d)[len(date) + 1:].replace("-", " ")

                urls = None
                if os.path.isfile('.' + d + '/urls.yml'):
                    urls = '.' + d + '/urls.yml'

                if extension in filetypes:
                    path = '.' + d + '/' + talk_filename + extension
                    if extension == '.link':
                        with open(path, 'r') as f:
                            path = f.read()
                    self.talks.append(
                        talk.Talk(title, date,
                                  path,
                                  extension, urls))
            except:
                pass
        return self.talks



"""
Script to write the html
"""
import crawl

class Index:
    """
    Class that is passed a list of talks and writes the required html
    """
    def __init__(self, talks, head=None, header=None, footer=None):
        self.talks = talks
        self.head = head
        self.header = header
        self.footer = footer

    def generate_output(self):
        self.out = """"""
        for f in [self.head, self.header]:
            try:
                self.out += open(f, 'r').read()
            except:
                pass
        self.out += """
<body>
<div class="page-content">
<div class="wrap">
<div class="home">
<ul class='posts'>"""
        for talk in sorted(self.talks, key=lambda x: x.date, reverse=True):
            self.out += """
<li>"""
            self.out += talk.html()
            self.out += """
</li>"""
        self.out += """
</ul>
</div>
</div>
</div>
"""
        if self.footer:
            self.out += open(self.footer, 'r').read()
        self.out += """
</body>
"""

    def write(self, output_file='index.html'):
        f = open(output_file, 'w')
        f.write(self.out)
        f.close()



from distutils.core import setup

setup(
    name='ghtalks',
    version='0.0.4',
    author='Vince Knight',
    url='https://github.com/drvinceknight/ghtalks',
    author_email=('vincent.knight@gmail.com'),
    packages=['ghtalks', 'ghtalks.tests'],
    scripts=['bin/gh-talks'],
    license='The MIT License (MIT)',
    description='Generate a static site with your slide decks',
    install_requires=[
        "pyyaml >= 3.11",
    ],
)



import unittest

import ghtalks

import os

class TestTalks(unittest.TestCase):

    def test_init(self):
        talk = ghtalks.Talk('talk 1', '2015-04-01', './directory', '.html')
        self.assertEquals(talk.title, 'talk 1')
        self.assertEquals(talk.date, '2015-04-01')
        self.assertEquals(talk.path, './directory')
        self.assertEquals(talk.file_type, '.html')

    def test_html(self):
        talk = ghtalks.Talk('talk 1', '2015-04-01', './directory/index.html', '.html')
        expected_out = """
<span class="post-date">2015-04-01 [.html]</span>\n<a class="post-link" href="./directory/index.html">talk 1</a>
"""
        self.assertEquals(talk.html(), expected_out)
